@@ -13,6 +13,9 @@
 #include <iostream>
 #include <fstream>
 
+namespace acng
+{
+
 class tSpecOpDetachable : public tSpecialRequest
 {
 public:
@@ -77,4 +80,6 @@ public:
 };
 #endif // DEBUG
 
+}
+
 #endif /* BGTASK_H_ */
