@@ -1,6 +1,10 @@
 #ifndef CONSERVER_H_
 #define CONSERVER_H_
 
+
+namespace acng
+{
+
 namespace conserver {
 
 /*! Prepares the connection handlers and internal things, binds, listens, etc.
@@ -14,4 +18,6 @@ void Shutdown();
 
 }
 
+}
+
 #endif /*CONSERVER_H_*/
