@@ -27,6 +27,9 @@ using namespace std;
 "</style></head><body>"
 #define LOG_DECO_END "</body></html>"
 
+namespace acng
+{
+
 // for start/stop and abort hint
 base_with_condition tSpecOpDetachable::g_StateCv;
 bool tSpecOpDetachable::g_sigTaskAbort=false;
@@ -43,16 +46,21 @@ tSpecOpDetachable::~tSpecOpDetachable()
 	checkforceclose(m_logFd);
 }
 
-cmstring& GetFooter()
+cmstring GetFooter()
 {
-	static mstring footer;
-	if(footer.empty())
-	{
-		footer = string("<hr><address>Server: ") + acfg::agentname +
-		"&nbsp;&nbsp;|&nbsp;&nbsp;<a href=\"https://flattr.com/thing/51105/Apt-Cacher-NG\">Flattr this!"
-    "</a>&nbsp;&nbsp;|&nbsp;&nbsp;<a href=\"http://www.unix-ag.uni-kl.de/~bloch/acng/\">Apt-Cacher NG homepage</a></address>";
-	}
-	return footer;
+        return mstring("<hr><address>Server: ") + cfg::agentname
+                + "&nbsp;&nbsp;|&nbsp;&nbsp;<a\nhref=\"https://flattr.com/thing/51105/Apt-Cacher-NG\">Flattr this!"
+                "</a>&nbsp;&nbsp;|&nbsp;&nbsp;<a\nhref=\"http://www.unix-ag.uni-kl.de/~bloch/acng/\">Apt-Cacher NG homepage</a></address>";
+}
+
+std::string to_base36(unsigned int val)
+{
+	static std::string base36 = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+	std::string result;
+	do {
+		result.insert(0, 1, base36[val % 36]);
+	} while (val /= 36);
+	return result;
 }
 
 /*
@@ -83,9 +91,9 @@ void tSpecOpDetachable::Run()
 	tSS deco;
 	const char *mark(nullptr);
 	if(m_szDecoFile &&
-			( deco.initFromFile((acfg::confdir+SZPATHSEP+m_szDecoFile).c_str())
-					|| (!acfg::suppdir.empty() &&
-							deco.initFromFile((acfg::suppdir+SZPATHSEP+m_szDecoFile).c_str()))))
+			( deco.initFromFile((cfg::confdir+SZPATHSEP+m_szDecoFile).c_str())
+					|| (!cfg::suppdir.empty() &&
+							deco.initFromFile((cfg::suppdir+SZPATHSEP+m_szDecoFile).c_str()))))
 	{
 		mark=::strchr(deco.rptr(), '~');
 		if(mark)
@@ -113,7 +121,7 @@ void tSpecOpDetachable::Run()
 			auto id = time(0);
 
 			logPath.clear();
-			logPath<<acfg::logdir<<CPATHSEP<< MAINT_PFX << id << ".log.html";
+			logPath<<cfg::logdir<<CPATHSEP<< MAINT_PFX << id << ".log.html";
 			m_reportStream.open(logPath.c_str(), ios::out);
 			if(m_reportStream.is_open())
 			{
@@ -152,7 +160,7 @@ void tSpecOpDetachable::Run()
 			if(m_logFd < 0)
 			{
 				logPath.clear();
-				logPath<<acfg::logdir<<CPATHSEP<<MAINT_PFX << other_id << ".log.html";
+				logPath<<cfg::logdir<<CPATHSEP<<MAINT_PFX << other_id << ".log.html";
 				m_logFd=open(logPath.c_str(), O_RDONLY|O_NONBLOCK);
 
 				if(m_logFd>=0)
@@ -193,7 +201,7 @@ void tSpecOpDetachable::Run()
 
 			SendFmt << "Maintenance task <b>" << GetTaskName()
 					<< "</b>, apt-cacher-ng version: " ACVERSION;
-			string link = "http://" + GetHostname() + ":" + acfg::port + "/" + acfg::reportpage;
+			string link = "http://" + GetHostname() + ":" + cfg::port + "/" + cfg::reportpage;
 			SendFmtRemote << " (<a href=\"" << m_parms.cmd << "&sigabort=" << rand()
 					<< "\">Cancel</a>)"
 					<< "\n<!--\n"
@@ -236,7 +244,7 @@ void tSpecOpDetachable::Run()
 					SendFmtRemote << err.first << ": <label>"
 							<< err.second.msg
 							<<  "<input type=\"checkbox\" name=\"kf\" value=\""
-							<< tSS::hex << err.second.id << tSS::dec
+							<< to_base36(err.second.id)
 							<< "\"</label>" << hendl;
 				}
 
@@ -260,7 +268,7 @@ void tSpecOpDetachable::Run()
 				SendChunkRemoteOnly(blob.data(), blob.size());
 			}
 
-			SendFmtRemote << "<br>\n<a href=\"/"<< acfg::reportpage<<"\">Return to main page</a>"
+			SendFmtRemote << "<br>\n<a href=\"/"<< cfg::reportpage<<"\">Return to main page</a>"
 					"</form>";
 			auto& f(GetFooter());
 						SendChunkRemoteOnly(f.data(), f.size());
@@ -286,8 +294,8 @@ void tSpecOpDetachable::DumpLog(time_t id)
 	if (id<=0)
 		return;
 
-	tSS path(acfg::logdir.length()+24);
-	path<<acfg::logdir<<CPATHSEP<<MAINT_PFX << id << ".log.html";
+	tSS path(cfg::logdir.length()+24);
+	path<<cfg::logdir<<CPATHSEP<<MAINT_PFX << id << ".log.html";
 	if (!reader.OpenFile(path))
 		SendChunkRemoteOnly(WITHLEN("Log not available"));
 	else
@@ -315,14 +323,21 @@ mstring tSpecOpDetachable::BuildCompressedDelFileCatalog()
 {
 	mstring ret;
 	tSS buf;
-	//auto hm = m_delCboxFilter.size();
+
+	// add the recent command, then the file records
+
+	auto addLine = [&buf](unsigned id, cmstring& s)
+		{
+		unsigned len=s.size();
+		buf.add((const char*) &id, sizeof(id))
+				.add((const char*) &len, sizeof(len))
+				.add(s.data(), s.length());
+		};
+	// don't care about the ID, compression will solve it
+	addLine(0, m_parms.cmd);
 	for(const auto& kv: m_pathMemory)
-	{
-		unsigned len=kv.first.size();
-		buf.add((const char*) &kv.second.id, sizeof(kv.second.id))
-		.add((const char*) &len, sizeof(len))
-		.add(kv.first.data(), kv.first.length());
-	}
+		addLine(kv.second.id, kv.first);
+
 	unsigned uncompSize=buf.size();
 	tSS gzBuf;
 	uLongf gzSize = compressBound(buf.size())+32; // extra space for length header
@@ -332,8 +347,7 @@ mstring tSpecOpDetachable::BuildCompressedDelFileCatalog()
 	if(Z_OK == compress((Bytef*) gzBuf.wptr(), &gzSize,
 			(const Bytef*)buf.rptr(), buf.size()))
 	{
-		ret = "<input type=\"hidden\" name=\"blob\" value=\"";
-//		ret += BytesToHexString((const uint8_t*) gzBuf.wptr(), (unsigned short) gzSize);
+		ret = "<input type=\"hidden\" name=\"blob\"\nvalue=\"";
 		ret += EncodeBase64(gzBuf.rptr(), (unsigned short) gzSize+sizeof(uncompSize));
 		ret += "\">";
 		return ret;
@@ -359,3 +373,5 @@ void tBgTester::Action()
 }
 
 #endif // DEBUG
+
+}
