@@ -29,8 +29,8 @@
 #include <unistd.h>
 
 #ifdef DEBUG
-#define DEBUGIDX
 #warning enable, and it will spam a lot!
+//#define DEBUGIDX
 //#define DEBUGSPAM
 #endif
 
@@ -38,11 +38,13 @@
 
 using namespace std;
 
+namespace acng
+{
 
 static cmstring dis("/binary-");
 static cmstring oldStylei18nIdx("/i18n/Index");
 static cmstring diffIdxSfx(".diff/Index");
-static const string relKey("/Release"), inRelKey("/InRelease");
+
 time_t m_gMaintTimeNow=0;
 
 static cmstring sPatchCombinedRel("_actmp/combined.diff");
@@ -145,7 +147,7 @@ bool cacheman::AddIFileCandidate(const string &sPathRel)
  		return false;
 
 	enumMetaType t;
-	if ( (rechecks::FILE_VOLATILE == rechecks::GetFiletype(sPathRel)
+	if ( (rex::FILE_VOLATILE == rex::GetFiletype(sPathRel)
 	// SUSE stuff, not volatile but also contains file index data
 	|| endsWithSzAr(sPathRel, ".xml.gz") )
 	&& (t=GuessMetaTypeFromURL(sPathRel)))
@@ -271,7 +273,7 @@ bool cacheman::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 
 #define GOTOREPMSG(x) {sErr = x; bSuccess=false; goto rep_dlresult; }
 
-	const acfg::tRepoData *pRepoDesc=nullptr;
+	const cfg::tRepoData *pRepoDesc=nullptr;
 	mstring sRemoteSuffix, sFilePathAbs(SABSPATH(sFilePathRel));
 
 	fileItemMgmt fiaccess;
@@ -286,7 +288,7 @@ bool cacheman::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 	if(!pFi)
 	{
 		fiaccess.PrepareRegisteredFileItemWithStorage(sFilePathRel, false);
-		pFi=fiaccess.get();
+		pFi=fiaccess.getFiPtr();
 	}
 	if (!pFi)
 	{
@@ -342,8 +344,8 @@ bool cacheman::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 		ldbg("Find backend for " << sFilePathRel << " parsed as host: "  << parserPath.sHost
 				<< " and path: " << parserPath.sPath << ", ok? " << bCachePathAsUriPlausible);
 
-		if(!acfg::stupidfs && bCachePathAsUriPlausible
-				&& 0 != (pRepoDesc = acfg::GetRepoData(parserPath.sHost))
+		if(!cfg::stupidfs && bCachePathAsUriPlausible
+				&& 0 != (pRepoDesc = cfg::GetRepoData(parserPath.sHost))
 				&& !pRepoDesc->m_backends.empty())
 		{
 			ldbg("will use backend mode, subdirectory is path suffix relative to backend uri");
@@ -400,8 +402,8 @@ bool cacheman::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 	// might still need a repo data description
 	if (pResolvedDirectUrl)
 	{
-		acfg::tRepoResolvResult repinfo;
-		acfg::GetRepNameAndPathResidual(*pResolvedDirectUrl, repinfo);
+		cfg::tRepoResolvResult repinfo;
+		cfg::GetRepNameAndPathResidual(*pResolvedDirectUrl, repinfo);
 		auto hereDesc = repinfo.repodata;
 		if(repinfo.repodata && !repinfo.repodata->m_backends.empty())
 		{
@@ -411,7 +413,7 @@ bool cacheman::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 		}
 	}
 
-	m_pDlcon->AddJob(pFi, pResolvedDirectUrl, pRepoDesc, &sRemoteSuffix, 0);
+	m_pDlcon->AddJob(pFi, pResolvedDirectUrl, pRepoDesc, &sRemoteSuffix, 0, cfg::REDIRMAX_DEFAULT);
 
 	m_pDlcon->WorkLoop();
 	if (pFi->WaitForFinish(nullptr) == fileitem::FIST_COMPLETE
@@ -447,6 +449,15 @@ bool cacheman::Download(cmstring& sFilePathRel, bool bIsVolatileFile,
 
 	rep_dlresult:
 
+	if(pFi)
+	{
+		auto dlCount = pFi->GetTransferCount();
+		static cmstring sInternal("[INTERNAL:");
+		// need to account both, this traffic as officially tracked traffic, and also keep the count
+		// separately for expiration about trade-off calculation
+		log::transfer(dlCount, 0, sInternal + GetTaskName() + "]", sFilePathRel, false);
+	}
+
 	if (bSuccess && bIsVolatileFile)
 		SetFlags(sFilePathRel).uptodate = true;
 
@@ -795,7 +806,7 @@ bool cacheman::Inject(cmstring &fromRel, cmstring &toRel,
 
 #ifdef DEBUG_FLAGS
 	bool nix = stmiss!=fromRel.find("debrep/dists/squeeze/non-free/binary-amd64/");
-	SendFmt<<"Replacing "<<toRel<<" with " << fromRel <<  "<br>\n";
+	SendFmt<<"Replacing "<<toRel<<" with " << fromRel <<  sBRLF;
 #endif
 
 	if(!infoFrom)
@@ -1251,7 +1262,7 @@ void cacheman::PatchOne(cmstring& pindexPathRel, const tStrDeq& siblings)
 			// ok, first patch of the sequence found
 			if(!pf.p)
 			{
-				::mkbasedir(sPatchCombinedAbs);
+				acng::mkbasedir(sPatchCombinedAbs);
 				// append mode!
 				pf.p=fopen(sPatchCombinedAbs.c_str(), "w");
 				if(!pf.p)
@@ -1303,8 +1314,8 @@ void cacheman::PatchOne(cmstring& pindexPathRel, const tStrDeq& siblings)
 
 			tSS cmd;
 			cmd << "cd '" << CACHE_BASE << "_actmp' && ";
-			auto act = acfg::suppdir + SZPATHSEP "acngtool";
-			if(!acfg::suppdir.empty() && 0==access(act.c_str(), X_OK))
+			auto act = cfg::suppdir + SZPATHSEP "acngtool";
+			if(!cfg::suppdir.empty() && 0==access(act.c_str(), X_OK))
 				cmd << "'" << act << "' patch patch.base " << sPatchCombinedAbs << " patch.result";
 			else
 				cmd << " red --silent patch.base < " << sPatchCombinedAbs;
@@ -1341,7 +1352,7 @@ void cacheman::PatchOne(cmstring& pindexPathRel, const tStrDeq& siblings)
 
 			FILE_RAII df;
 			DelTree(SABSPATH("_actmp"));
-			::mkbasedir(sPatchInputAbs);
+			acng::mkbasedir(sPatchInputAbs);
 			df.p = fopen(sPatchInputAbs.c_str(), "w");
 			if(!df.p)
 			{
@@ -1436,7 +1447,7 @@ void cacheman::UpdateVolatileFiles()
 		auto dbgDump = [&](const char *msg, int pfx) {
 			tSS printBuf;
 			printBuf << "#########################################################################<br>\n"
-					<< "## " <<  msg  << "<br>\n"
+					<< "## " <<  msg  << sBRLF
 					<< "#########################################################################<br>\n";
 			for(const auto& cp : idxGroups)
 			{
@@ -1458,6 +1469,13 @@ void cacheman::UpdateVolatileFiles()
 
 	MTLOGDEBUG("<br><br><b>STARTING ULTIMATE INTELLIGENCE</b><br><br>");
 
+	if(m_bSkipIxUpdate)
+	{
+		SendFmt << "<span class=\"ERROR\">"
+				"Warning: Online Activity is disabled, some update errors might be not recoverable without it"
+				"<br></span>\n";
+	}
+
 	// this runs early with the state that is present on disk, before updating any file,
 	// since it deals with the "reality" in the cache
 
@@ -1492,7 +1510,7 @@ void cacheman::UpdateVolatileFiles()
 
 	{
 		std::unordered_set<std::string> oldReleaseFiles;
-		auto baseFolder = acfg::cacheDirSlash + acfg::privStoreRelSnapSufix;
+		auto baseFolder = cfg::cacheDirSlash + cfg::privStoreRelSnapSufix;
 		IFileHandler::FindFiles(baseFolder, [&baseFolder, &oldReleaseFiles, this](cmstring &sPath, const struct stat &st)
 				-> bool {
 			oldReleaseFiles.emplace(sPath.substr(baseFolder.size() + 1));
@@ -2156,7 +2174,7 @@ void cacheman::ProcessSeenIndexFiles(std::function<void(tRemoteFileInfo)> pkgHan
 
 		//bool bNix=(it->first.find("experimental/non-free/binary-amd64/Packages.xz") != stmiss);
 
-		SendChunk(string("Parsing metadata in ")+path2att.first+"<br>\n");
+		SendChunk(string("Parsing metadata in ")+path2att.first+sBRLF);
 
 		if( ! ParseAndProcessMetaFile(pkgHandler, path2att.first, itype))
 		{
@@ -2165,7 +2183,7 @@ void cacheman::ProcessSeenIndexFiles(std::function<void(tRemoteFileInfo)> pkgHan
 				m_nErrorCount++;
 				SendChunk("<span class=\"ERROR\">An error occurred while reading this file, some contents may have been ignored.</span>\n");
 				AddDelCbox(path2att.first, "Index data processing error");
-				SendChunk("<br>\n");
+				SendChunk(sBRLF);
 			}
 			continue;
 		}
@@ -2200,9 +2218,24 @@ void cacheman::AddDelCbox(cmstring &sFileRel, cmstring& reason, bool bIsOptional
 }
 void cacheman::TellCount(unsigned nCount, off_t nSize)
 {
-	SendFmt << "<br>\n" << nCount <<" package file(s) marked "
+	SendFmt << sBRLF << nCount <<" package file(s) marked "
 			"for removal in few days. Estimated disk space to be released: "
-			<< offttosH(nSize) << ".<br>\n<br>\n";
+			<< offttosH(nSize) << "." << sBRLF << sBRLF;
+}
+
+mstring cacheman::AddLookupGetKey(cmstring &sFilePathRel, cmstring& errorReason)
+{
+	unsigned id = m_pathMemory.size();
+	auto it = m_pathMemory.find(sFilePathRel);
+	if(it==m_pathMemory.end())
+		m_pathMemory[sFilePathRel] = {errorReason, id};
+	else
+		id = it->second.id;
+
+	mstring ret(WITHLEN(" name=\"kf\" value=\""));
+	ret +=to_base36(id);
+	ret += "\"";
+	return ret;
 }
 
 void cacheman::PrintStats(cmstring &title)
@@ -2319,13 +2352,14 @@ bool cacheman::_checkSolidHashOnDisk(cmstring& hexname,
 void cacheman::BuildCacheFileList()
 {
 	//dump_proc_status();
-	IFileHandler::DirectoryWalk(acfg::cachedir, this);
+	IFileHandler::DirectoryWalk(cfg::cachedir, this);
 	//dump_proc_status();
 }
 
 bool cacheman::ProcessByHashReleaseFileRestoreFiles(cmstring& releasePathRel, cmstring& stripPrefix)
 {
 	int errors = 0;
+
 	return ParseAndProcessMetaFile([this, &errors, &stripPrefix](const tRemoteFileInfo &entry) -> void
 	{
 		// ignore, those files are empty and are likely to report false positives
@@ -2348,7 +2382,7 @@ bool cacheman::ProcessByHashReleaseFileRestoreFiles(cmstring& releasePathRel, cm
 		Cstat wantedState(wantedPathAbs);
 		string solidPathRel, solidPathAbs;
 		// lazy construction for the check below
-		if(wantedState.st_size != entry.fpr.size)
+		if(!wantedState || wantedState.st_size != entry.fpr.size)
 		{
 			solidPathRel = entry.sDirectory.substr(stripPrefix.size()) + "by-hash/" +
 									GetCsNameReleaseFile(entry.fpr.csType) + '/' + hexname;
@@ -2361,7 +2395,8 @@ bool cacheman::ProcessByHashReleaseFileRestoreFiles(cmstring& releasePathRel, cm
 		if(!wantedState || (wantedState.st_size != entry.fpr.size
 				&& (contentMatch = entry.fpr.CheckFile(solidPathAbs))))
 		{
-			SendFmt << "Restoring virtual file " << wantedPathRel
+			if(m_bVerbose)
+				SendFmt << "Restoring virtual file " << wantedPathRel
 					<< " (equal to " << solidPathRel << ")" << hendl;
 
 			// return with increased count if error happens
@@ -2370,17 +2405,29 @@ bool cacheman::ProcessByHashReleaseFileRestoreFiles(cmstring& releasePathRel, cm
 			header h;
 			// load by-hash header, check URL, rewrite URL, copy the stuff over
 			if(!h.LoadFromFile(SABSPATH(solidPathRel) + ".head") || ! h.h[header::XORIG])
+				{
+				if(m_bVerbose)
+					SendFmt << "Couldn't read " << SABSPATH(solidPathRel) << ".head<br>";
 				return;
+				}
 			string origin(h.h[header::XORIG]);
 			tStrPos pos = origin.rfind("by-hash/");
 			if(pos == stmiss)
-				return;
+			{
+			if(m_bVerbose)
+				SendFmt << SABSPATH(solidPathRel) << " is not from by-hash folder<br>";
+			return;
+			}
 			h.set(header::XORIG, origin.substr(0, pos) + entry.sFileName);
 			// most servers report crap type on by-hash files, use generic one
 			h.set(header::CONTENT_TYPE, "octet/stream");
 			//	should be ok				h.set(header::CONTENT_LENGTH, entry.fpr.size)
 			if(!Inject(solidPathRel, wantedPathRel, false, &h, false))
-				return;
+			{
+			if(m_bVerbose)
+				SendChunk("Couldn't install<br>");
+			return;
+			}
 			auto& flags = SetFlags(wantedPathRel);
 			if(flags.vfile_ondisk)
 				flags.uptodate = true;
@@ -2396,7 +2443,7 @@ bool cacheman::FixMissingByHashLinks(std::unordered_set<std::string> &oldRelease
 	bool ret = true;
 
 	// path of side store with trailing slash relativ to cache folder
-	auto srcPrefix(acfg::privStoreRelSnapSufix + sPathSep);
+	auto srcPrefix(cfg::privStoreRelSnapSufix + sPathSep);
 
 	for(const auto& snapPathInXstore: oldReleaseFiles)
 	{
@@ -2405,7 +2452,9 @@ bool cacheman::FixMissingByHashLinks(std::unordered_set<std::string> &oldRelease
 		// path relative to cache folder
 		if(!ProcessByHashReleaseFileRestoreFiles(snapPathInXstore, srcPrefix))
 		{
-			SendFmt << "Error at " << snapPathInXstore << hendl;
+			SendFmt << "There were error(s) processing " << snapPathInXstore << ", ignoring..."<< hendl;
+			if(!m_bVerbose)
+				SendChunk("Enable verbosity to see more");
 			return ret;
 		}
 #ifdef DEBUGIDX
@@ -2451,3 +2500,5 @@ tStrDeq cacheman::GetGoodReleaseFiles()
 	for(const auto& kv: t) ret.emplace_back(kv.first+kv.second);
 	return ret;
 }
+
+}
