@@ -24,14 +24,21 @@ using namespace std;
 #warning Unlinking parts defused
 #endif
 
-#define TIMEEXPIRED(t) (t < (m_gMaintTimeNow-acfg::extreshhold*86400))
-#define TIME_AGONY (m_gMaintTimeNow-acfg::extreshhold*86400)
+#define TIMEEXPIRED(t) (t < (m_gMaintTimeNow-acng::cfg::extreshhold*86400))
+#define TIME_AGONY (m_gMaintTimeNow-acng::cfg::extreshhold*86400)
 
 #define FNAME_PENDING "_expending_dat"
 #define FNAME_DAMAGED "_expending_damaged"
 #define sFAIL_INI SABSPATH("_exfail_cnt")
 #define FAIL_INI sFAIL_INI.c_str()
 
+namespace acng
+{
+
+// represents the session's current incomming data count at the moment when expr. was run last time
+// it's needed to correct the considerations based on the active session's download stats
+off_t lastCurrentDlCount(0);
+
 void expiration::HandlePkgEntry(const tRemoteFileInfo &entry)
 {
 #ifdef DEBUGSPAM
@@ -185,7 +192,7 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry)
 				// IO error? better keep it for now, not sure how to deal with it
 				SendFmt << ECLASS "An error occurred while checksumming "
 				<< sPathRel << ", leaving as-is for now.";
-				aclog::err(tSS() << "Error reading " << sPathAbs );
+				log::err(tSS() << "Error reading " << sPathAbs );
 				AddDelCbox(sPathRel, "IO error");
 				SendFmt<<CLASSEND;
 				return false;
@@ -215,7 +222,7 @@ ERROR: size mismatch on debrep/dists/jessie/contrib/i18n/Translation-en.bz2 (add
 repro: check by path, after a minor distro upgrade
 				 */
 				// don't really care but let the user truncate
-				if(rechecks::GetFiletype(sPathRel) == rechecks::FILE_VOLATILE)
+				if(rex::GetFiletype(sPathRel) == rex::FILE_VOLATILE)
 					return report_weird_volatile(lenFromStat);
 
 				SendFmt << ECLASS << "size mismatch on " << sPathRel;
@@ -275,7 +282,7 @@ repro: check by path, after a minor distro upgrade
 	// and in general, all kinds of index files shall be checked at the particular location since
 	// there are too many identical names spread between different repositories
 	bool byPath = (m_bByPath || entry.sFileName == sIndex ||
-			rechecks::Match(entry.sDirectory + entry.sFileName, rechecks::FILE_VOLATILE));
+			rex::Match(entry.sDirectory + entry.sFileName, rex::FILE_VOLATILE));
 	if(byPath)
 	{
 		// compare full paths (physical vs. remote) with their real paths
@@ -297,12 +304,12 @@ repro: check by path, after a minor distro upgrade
 // of gaps in the "package history" when proceeded in linear fashion.
 inline void expiration::DropExceptionalVersions()
 {
-    if(m_trashFile2dir2Info.empty() || !acfg::keepnver)
+    if(m_trashFile2dir2Info.empty() || !cfg::keepnver)
     	return;
     if(system("dpkg --version >/dev/null 2>&1"))
 	{
 		SendFmt << "dpkg not available on this system, cannot identify latest versions to keep "
-				"only " << acfg::keepnver << " of them.";
+				"only " << cfg::keepnver << " of them.";
 		return;
     }
     struct tPkgId
@@ -344,9 +351,9 @@ inline void expiration::DropExceptionalVersions()
     auto procGroup = [&]()
 		{
     	// if more than allowed, keep the highest versions for sure, others are expired as usual
-    	if(version2trashGroup.size() > (uint) acfg::keepnver)
+    	if(version2trashGroup.size() > (uint) cfg::keepnver)
         	std::sort(version2trashGroup.begin(), version2trashGroup.end());
-    	for(unsigned i=0; i<version2trashGroup.size() && i<uint(acfg::keepnver); i++)
+    	for(unsigned i=0; i<version2trashGroup.size() && i<uint(cfg::keepnver); i++)
     		for(auto& j: * version2trashGroup[i].group)
     			j.second.nLostAt=m_gMaintTimeNow;
     	version2trashGroup.clear();
@@ -404,7 +411,7 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 			string sPathRel = dir_props.first + fileGroup.first;
 			auto& desc = dir_props.second;
 			DBGQLOG("Checking " << sPathRel);
-			using namespace rechecks;
+			using namespace rex;
 
 			if (ContHas(m_forceKeepInTrash, sPathRel))
 			{
@@ -428,11 +435,15 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 
 			if (bPurgeNow || TIMEEXPIRED(dir_props.second.nLostAt))
 			{
-				SendFmt << "Removing " << sPathRel << "<br>\n";
 
 #ifdef ENABLED
-				::unlink(sPathAbs.c_str());
-				::unlink((sPathAbs + ".head").c_str());
+				SendFmt << "Removing " << sPathRel;
+				if(::unlink(sPathAbs.c_str()))
+					SendChunk(tErrnoFmter("<span class=\"ERROR\"> [ERROR] ")+"</span>");
+				SendFmt << sBRLF << "Removing " << sPathRel << ".head";
+				if(::unlink((sPathAbs + ".head").c_str()))
+					SendChunk(tErrnoFmter("<span class=\"ERROR\"> [ERROR] ")+"</span>");
+				SendChunk(sBRLF);
 				::rmdir(SZABSPATH(dir_props.first));
 #endif
 			}
@@ -441,7 +452,7 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 				SendFmt << "Tagging " << sPathRel;
 				if (m_bVerbose)
 					SendFmt << " (t-" << (m_gMaintTimeNow - desc.nLostAt) / 3600 << "h)";
-				SendChunk("<br>\n");
+				SendChunk(sBRLF);
 
 				nCount++;
 				tagSpace += desc.fpr.size;
@@ -476,6 +487,27 @@ void expiration::Action()
 		return;
 	}
 
+	bool tradeOffCheck = cfg::exstarttradeoff && !StrHas(m_parms.cmd, "ignoreTradeOff") && !m_bByChecksum;
+
+	off_t newLastIncommingOffset = 0;
+
+	if(tradeOffCheck)
+	{
+		newLastIncommingOffset = log::GetCurrentCountersInOut().first;
+
+		auto haveIncomming = newLastIncommingOffset - lastCurrentDlCount
+				+ log::GetOldCountersInOut(true).first;
+		if(haveIncomming < cfg::exstarttradeoff)
+		{
+			SendFmt << "Expiration suppressed due to costs-vs.-benefit considerations "
+					"(see exStartTradeOff setting, " << offttosH(haveIncomming) <<
+					" vs. " << offttosH(cfg::exstarttradeoff)
+					<< " (<a href=\"" << this->m_parms.cmd << "&ignoreTradeOff=iTO\">Override this check now</a>)"
+					<< sBRLF;
+			return;
+		}
+	}
+
 	m_bIncompleteIsDamaged=StrHas(m_parms.cmd, "incomAsDamaged");
 	m_bScanVolatileContents=StrHas(m_parms.cmd, "scanVolatile");
 
@@ -488,7 +520,7 @@ void expiration::Action()
 #if 0 //def DEBUG
 	for(auto& i: m_trashFile2dir2Info)
 	{
-		SendFmt << "<br>File: " << i.first <<"<br>\n";
+		SendFmt << "<br>File: " << i.first <<sBRLF;
 		for(auto& j: i.second)
 			 SendFmt << "Dir: " << j.first << " [ "<<j.second.fpr.size << " / " << j.second.nLostAt << " ]<br>\n";
 	}
@@ -545,10 +577,18 @@ void expiration::Action()
 
 	DelTree(CACHE_BASE+"_actmp");
 
+	TrimFiles();
+
 	PrintStats("Allocated disk space");
 
 	SendChunk("<br>Done.<br>");
 
+	if(tradeOffCheck)
+	{
+		lastCurrentDlCount = newLastIncommingOffset;
+		log::ResetOldCounters();
+	}
+
 	save_fail_count:
 
 	if (m_nErrorCount <= 0)
@@ -597,7 +637,7 @@ void expiration::ListExpiredFiles()
 					continue;
 
 				cnt++;
-				this->SendChunk(rel + "<br>\n");
+				this->SendChunk(rel + sBRLF);
 				nSpace += sz;
 
 				sz = GetFileSize(abspath + ".head", -2);
@@ -617,6 +657,38 @@ void expiration::ListExpiredFiles()
 	return;
 }
 
+void expiration::TrimFiles()
+{
+	if(m_oversizedFiles.empty())
+		return;
+	auto now=GetTime();
+	SendFmt << "<b>Trimming cache files (" << m_oversizedFiles.size() <<")</b>" << sBRLF;
+	for(const auto& fil: m_oversizedFiles)
+	{
+		// still there and not changed?
+		Cstat stinfo(fil);
+		if(!stinfo)
+			continue;
+		if(now - 86400 > stinfo.st_mtim.tv_sec)
+		{
+//			SendFmt << "let's truncate " << fil << " to " << stinfo.st_size << "<br>";
+			// it's unlikely to be accessed but better protect it
+			fileItemMgmt item;
+			item.PrepareRegisteredFileItemWithStorage(fil, true);
+			if(!item)
+				continue;
+			lockguard g(item.m_ptr.get());
+			off_t nix;
+			if(item.m_ptr->GetStatusUnlocked(nix) >= fileitem::FIST_DLGOTHEAD)
+				continue;
+
+			if(0 != truncate(fil.c_str(), stinfo.st_size))
+				SendFmt << "Error at " << fil << " (" << tErrnoFmter() << ")" << sBRLF;
+
+		}
+	}
+}
+
 void expiration::HandleDamagedFiles()
 {
 	filereader f;
@@ -633,30 +705,30 @@ void expiration::HandleDamagedFiles()
 
 			if(this->m_parms.type == workExPurgeDamaged)
 			{
-				SendFmt << "Removing " << s << "<br>\n";
+				SendFmt << "Removing " << s << sBRLF;
 				unlink(SZABSPATH(s));
 				unlink(SZABSPATH(s+".head"));
 			}
 			else if(this->m_parms.type == workExTruncDamaged)
 			{
-				SendFmt << "Truncating " << s << "<br>\n";
+				SendFmt << "Truncating " << s << sBRLF;
 				ignore_value(truncate(SZABSPATH(s), 0));
 			}
 			else
-				SendFmt << s << "<br>\n";
+				SendFmt << s << sBRLF;
 		}
 	return;
 }
 
 void expiration::PurgeMaintLogs()
 {
-	tStrDeq logs = ExpandFilePattern(acfg::logdir + SZPATHSEP MAINT_PFX "*.log*");
+	tStrDeq logs = ExpandFilePattern(cfg::logdir + SZPATHSEP MAINT_PFX "*.log*");
 	if (logs.size() > 2)
 		SendChunk(WITHLEN(
 				"Found required cleanup tasks: purging maintenance logs...<br>\n"));
 	for (const auto &s: logs)
 	{
-		time_t id = atoofft(s.c_str() + acfg::logdir.size() + 7);
+		time_t id = atoofft(s.c_str() + cfg::logdir.size() + 7);
 		//cerr << "id ist: "<< id<<endl;
 		if (id == GetTaskId())
 			continue;
@@ -670,7 +742,7 @@ void expiration::PurgeMaintLogs()
 		SendChunk(WITHLEN("Removing deprecated files...<br>\n"));
 		for(const auto &s: m_killBill)
 		{
-			SendChunk(s+"<br>\n");
+			SendChunk(s+sBRLF);
 			::unlink(SZABSPATH(s));
 		}
 	}
@@ -687,6 +759,27 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 		return false;
 
 	ProgTell();
+	auto diffMoreThan = [&stinfo](blkcnt_t diff){
+		return stinfo.st_blocks > diff && stinfo.st_size/512 < (stinfo.st_blocks - diff);
+	};
+
+	// detect invisible holes at the end of files (side effect of properly incorrect hidden allocation)
+	// allow some tolerance of about 10kb, should cover all page alignment effects
+	if(diffMoreThan(20))
+	{
+		auto now=GetTime();
+		if(now - 86400 > stinfo.st_mtim.tv_sec)
+		{
+			m_oversizedFiles.emplace_back(sPathAbs);
+
+			// don't spam unless the user wants it and the size is really large
+			if(m_bVerbose || diffMoreThan(40))
+			{
+				SendFmt << "Trailing allocated space on " << sPathAbs << " (" << stinfo.st_blocks <<
+						" blocks, expected: ~" << (stinfo.st_size/512  + 1) <<"), will be trimmed later<br>";
+			}
+		}
+	}
 
 	string sPathRel(sPathAbs, CACHE_BASE_LEN);
 	DBGQLOG(sPathRel);
@@ -726,7 +819,7 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 		attr.space += stinfo.st_size;
 		attr.forgiveDlErrors = endsWith(sPathRel, sslIndex);
 	}
-	else if (rechecks::Match(sPathRel, rechecks::FILE_VOLATILE))
+	else if (rex::Match(sPathRel, rex::FILE_VOLATILE))
 		return true; // cannot check volatile files properly so don't care
 
 	// ok, split to dir/file and add to the list
@@ -746,9 +839,9 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 void expiration::LoadHints()
 {
 	filereader reader;
-	if(!reader.OpenFile(acfg::confdir+SZPATHSEP+"ignore_list"))
+	if(!reader.OpenFile(cfg::confdir+SZPATHSEP+"ignore_list"))
 	{
-		if(acfg::suppdir.empty() || !reader.OpenFile(acfg::suppdir+SZPATHSEP+"ignore_list"))
+		if(cfg::suppdir.empty() || !reader.OpenFile(cfg::suppdir+SZPATHSEP+"ignore_list"))
 				return;
 	}
 	string sTmp;
@@ -818,8 +911,8 @@ void expiration::LoadPreviousData(bool bForceInsert)
 		char buf[200];
 
 		// map to to wait
-		int nMax=acfg::extreshhold-((now-oldest)/86400);
-		int nMin=acfg::extreshhold-((now-newest)/86400);
+		int nMax=cfg::extreshhold-((now-oldest)/86400);
+		int nMin=cfg::extreshhold-((now-newest)/86400);
 
 		snprintf(buf, _countof(buf), "Previously detected: %lu rotten package file(s), "
 				"to be deleted in about %d-%d day(s)<br>\n",
@@ -836,7 +929,7 @@ inline bool expiration::CheckAndReportError()
 	if (m_nErrorCount > 0 && m_bErrAbort)
 	{
 		SendFmt << sAbortMsg;
-		if(m_nPrevFailCount+(m_nErrorCount>0) > acfg::exsupcount)
+		if(m_nPrevFailCount+(m_nErrorCount>0) > cfg::exsupcount)
 			SendFmt << "\n<!--\n" maark << int(ControLineType::Error) << "Errors found, aborting expiration...\n-->\n";
 		return true;
 	}
@@ -865,3 +958,5 @@ bool expiration::_QuickCheckSolidFileOnDisk(cmstring& sPathRel)
 		return false;
 	return it->second.find(dir) != it->second.end();
 }
+
+}
