@@ -18,8 +18,11 @@
 #ifdef HAVE_TOMCRYPT
 #include <tomcrypt.h>
 #endif
+
 using namespace std;
 
+namespace acng
+{
 cmstring sPathSep(SZPATHSEP);
 cmstring sPathSepUnix(SZPATHSEPUNIX);
 #ifndef MINIBUILD
@@ -206,7 +209,7 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw, bool unescape)
 	else if(0==strncasecmp(url.c_str(), "https://", 8))
 	{
 #ifndef HAVE_SSL
-	aclog::err("E_NOTIMPLEMENTED: SSL");
+	log::err("E_NOTIMPLEMENTED: SSL");
 	return false;
 #else
 		hStart=8;
@@ -249,11 +252,11 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw, bool unescape)
 	
 	sHost=url.substr(hStart, hEndSuc-hStart);
 
-	// credentials might in there, strip them of
+	// credentials might be in there, strip them off
 	l=sHost.rfind('@');
 	if(l!=mstring::npos)
 	{
-		sUserPass=sHost.substr(0, l);
+		sUserPass = UrlUnescape(sHost.substr(0, l));
 		sHost.erase(0, l+1);
 	}
 
@@ -272,16 +275,22 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw, bool unescape)
 	
 	strip_ipv6_junk:
 	
+	bool host_appears_to_be_ipv6 = false;
 	if(sHost[0]=='[')
 	{
+		host_appears_to_be_ipv6 = true;
 		bCheckBrac=true;
 		sHost.erase(0,1);
 	}
 	
-	if(sHost[sHost.length()-1]==']')
+	if (sHost[sHost.length()-1] == ']') {
+		bCheckBrac = !bCheckBrac;
 		sHost.erase(sHost.length()-1);
-	else if(bCheckBrac) // must have been present here
+	}
+	if (bCheckBrac) // Unmatched square brackets.
 		return false;
+	if (!host_appears_to_be_ipv6)
+		sHost = UrlUnescape(sHost);
 	
 	return true;
 	
@@ -576,6 +585,63 @@ mstring UrlEscape(cmstring &s)
 	return ret;
 }
 
+/* From RFC3986 [0]:
+ *
+ *      sub-delims  = "!" / "$" / "&" / "'" / "(" / ")"
+ *                  / "*" / "+" / "," / ";" / "="
+ *
+ *      unreserved  = ALPHA / DIGIT / "-" / "." / "_" / "~"
+ *
+ *      authority   = [ userinfo "@" ] host [ ":" port ]
+ *
+ *      userinfo    = *( unreserved / pct-encoded / sub-delims / ":" )
+ *
+ * 0: https://www.ietf.org/rfc/rfc3986.txt
+ */
+static bool is_allowed_unencoded_userinfo_char(char c)
+{
+	switch (c) {
+	// unreserved:
+	case 'A': case 'B': case 'C': case 'D': case 'E': case 'F': case 'G':
+	case 'H': case 'I': case 'J': case 'K': case 'L': case 'M': case 'N':
+	case 'O': case 'P': case 'Q': case 'R': case 'S': case 'T': case 'U':
+	case 'V': case 'W': case 'X': case 'Y': case 'Z':
+	case 'a': case 'b': case 'c': case 'd': case 'e': case 'f': case 'g':
+	case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
+	case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
+	case 'v': case 'w': case 'x': case 'y': case 'z':
+	case '0': case '1': case '2': case '3': case '4':
+	case '5': case '6': case '7': case '8': case '9':
+	case '-': case '.': case '_': case '~':
+	// sub-delims:
+	case '!': case '$': case '&': case '\'': case '(': case ')':
+	case '*': case '+': case ',': case ';':  case '=':
+	// colon:
+	case ':':
+		return true;
+	default:
+		return false;
+	}
+}
+
+mstring UserinfoEscape(cmstring &s)
+{
+	mstring ret;
+	ret.reserve(s.size());
+
+	for (const auto& c : s) {
+		if (is_allowed_unencoded_userinfo_char(c)) {
+			ret += c;
+		} else {
+			char pct_encoded[4] = { '%', h2t_map[uint8_t(c) >> 4],
+			                             h2t_map[uint8_t(c) & 0x0f], '\0'};
+			ret += pct_encoded;
+		}
+	}
+
+	return ret;
+}
+
 mstring DosEscape(cmstring &s)
 {
 	mstring ret;
@@ -653,6 +719,9 @@ string EncodeBase64(LPCSTR data, unsigned len)
 	unsigned char_count=0;
 	char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 	string out;
+	//int newLineAfter = textWidth * 3 / 4;
+	//int linePos=0;
+
 	for(auto p=data; p<data+len; ++p)
 	{
 		uint8_t c=*p;
@@ -678,6 +747,13 @@ string EncodeBase64(LPCSTR data, unsigned len)
 		}
 		else
 			bits <<= 8;
+/*
+		if(newLineAfter > 0 && linePos++ >= newLineAfter)
+		{
+			linePos = 0;
+			out+='\n';
+		}
+		*/
 	}
 	if (char_count != 0)
 	{
@@ -765,7 +841,7 @@ mstring UrlUnescape(cmstring &from)
 	UrlUnescapeAppend(from, ret);
 	return ret;
 }
-mstring DosEscape(cmstring &s);
+//mstring DosEscape(cmstring &s);
 // just the bare minimum to make sure the string does not break HTML formating
 mstring html_sanitize(cmstring& in)
 {
@@ -789,6 +865,9 @@ mstring ltos(long n)
 	return mstring(buf, len);
 }
 
+/**
+ * Human friendly presentation of numbers, with units and only few bytes after comma
+ */
 mstring offttosH(off_t n)
 {
 	LPCSTR  pref[]={"", " KiB", " MiB", " GiB", " TiB", " PiB", " EiB"};
@@ -804,6 +883,41 @@ mstring offttosH(off_t n)
 	return "INF";
 }
 
+mstring offttosHdotted(off_t n)
+{
+	mstring ret(to_string(n));
+	auto pos = ret.size()-1;
+	for(unsigned i=1; pos > 0; ++i, --pos)
+		if(0 == i%3)
+			ret.insert(pos, ".");
+	return ret;
+}
+
+
+//template<typename charp>
+off_t strsizeToOfft(const char *sizeString) // XXX: if needed... charp sizeString, charp *next)
+{
+	char *inext(0);
+	auto val = strtoull(sizeString, &inext, 10);
+	if(!val) return 0;
+	if(!*inext) return val; // full length
+	// trim
+	while(*inext && isspace((unsigned)*inext)) ++inext;
+	switch(*inext)
+	{
+	case 'k': return val * 1000;
+	case 'm': return val * 1000000;
+	case 'g': return val * 1000000*1000;
+	case 'p': return val * 1000000*1000000;
+
+	case 'K': return val * 1024;
+	case 'M': return val * 1024*1024;
+	case 'G': return val * 1024*1024*1024;
+	case 'P': return val * 1024*1024*1024*1024;
+	}
+	return val;
+}
+
 void replaceChars(mstring &s, LPCSTR szBadChars, char goodChar)
 {
 	for(mstring::iterator p=s.begin();p!=s.end();p++)
@@ -877,3 +991,5 @@ bool scaseequals(cmstring& a, cmstring& b)
             return false;
     return true;
 }
+
+}
